package com.denizenscript.clientizen.objects;

import com.denizenscript.clientizen.util.Utilities;
import com.denizenscript.denizencore.objects.*;
import com.denizenscript.denizencore.objects.core.ElementTag;
import com.denizenscript.denizencore.objects.properties.PropertyParser;
import com.denizenscript.denizencore.tags.Attribute;
import com.denizenscript.denizencore.tags.ObjectTagProcessor;
import com.denizenscript.denizencore.tags.TagContext;
import com.denizenscript.denizencore.utilities.CoreUtilities;
import com.denizenscript.denizencore.utilities.debugging.Debug;
import net.minecraft.class_1792;
import net.minecraft.class_2248;
import net.minecraft.class_2680;
import net.minecraft.class_2960;
import net.minecraft.class_7923;

public class MaterialTag implements ObjectTag, Adjustable {

    // Needs to match the server-side impl, which uses bukkit's all-in-one Material enum
    public class_2680 state;
    public class_1792 item;

    public MaterialTag(class_2680 state) {
        this.state = state;
    }

    public MaterialTag(class_2248 block) {
        this.state = block.method_9564();
    }

    public MaterialTag(class_1792 item) {
        this.item = item;
    }

    @Fetchable("m")
    public static MaterialTag valueOf(String string, TagContext context) {
        if (string == null) {
            return null;
        }
        if (ObjectFetcher.isObjectWithProperties(string)) {
            return ObjectFetcher.getObjectFromWithProperties(ClientizenObjectRegistry.TYPE_MATERIAL, string, context);
        }
        if (string.startsWith("m@")) {
            string = string.substring("m@".length());
        }
        class_2960 identifier = class_2960.method_12829(string);
        if (identifier == null) {
            if (context == null || context.showErrors()) {
                Debug.echoError("valueOf MaterialTag returning null, invalid material/item name or identifier specified. For input: " + string);
            }
            return null;
        }
        MaterialTag material = class_7923.field_41175.method_17966(identifier).map(MaterialTag::new)
                .orElseGet(() -> class_7923.field_41178.method_17966(identifier).map(MaterialTag::new).orElse(null));
        if (material != null) {
            return material;
        }
        if (context == null || context.showErrors()) {
            Debug.echoError("valueOf MaterialTag returning null, invalid material/item specified. For input: " + string);
        }
        return null;
    }

    public static boolean matches(String string) {
        if (string.startsWith("m@")) {
            return true;
        }
        return valueOf(string, CoreUtilities.noDebugContext) != null;
    }

    public String getName() {
        return Utilities.idToString(state != null ? class_7923.field_41175.method_10221(state.method_26204()) : class_7923.field_41178.method_10221(item));
    }

    public static void register() {
        PropertyParser.registerPropertyTagHandlers(MaterialTag.class, tagProcessor);

        tagProcessor.registerTag(ElementTag.class, "name", (attribute, object) -> {
            return new ElementTag(object.getName());
        });
    }

    public static final ObjectTagProcessor<MaterialTag> tagProcessor = new ObjectTagProcessor<>();

    @Override
    public ObjectTag getObjectAttribute(Attribute attribute) {
        return tagProcessor.getObjectAttribute(this, attribute);
    }

    @Override
    public void adjust(Mechanism mechanism) {
        tagProcessor.processMechanism(this, mechanism);
    }

    @Override
    public void applyProperty(Mechanism mechanism) {
        adjust(mechanism);
    }

    @Override
    public String identify() {
        return "m@" + getName() + PropertyParser.getPropertiesString(this);
    }

    @Override
    public String identifySimple() {
        return "m@" + getName();
    }

    @Override
    public String debuggable() {
        return "<LG>m@<Y>" + getName() + PropertyParser.getPropertiesDebuggable(this);
    }

    @Override
    public String toString() {
        return identify();
    }

    @Override
    public boolean isUnique() {
        return false;
    }

    @Override
    public ObjectTag duplicate() {
        return state != null ? new MaterialTag(state) : new MaterialTag(item);
    }

    private String prefix = "Material";

    @Override
    public String getPrefix() {
        return prefix;
    }

    @Override
    public ObjectTag setPrefix(String prefix) {
        this.prefix = prefix;
        return this;
    }
}
